//
//  AnimationChainAppDelegate.m
//  AnimationChain
//
//  Copyright Matt Rajca 2009. All rights reserved.
//

#import "AnimationChainAppDelegate.h"

#import "AnimationStepOperation.h"

@implementation AnimationChainAppDelegate

#define kSquareWidth 64.0f
#define kSquareCornerRadius 8.0f

@synthesize window, customView;

- (id)init {
	self = [super init];
	if (self) {
		chainQueue = [[NSOperationQueue alloc] init];
		[chainQueue setMaxConcurrentOperationCount:1];
	}
	return self;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
	CALayer *rootLayer = [customView layer];
	
	squareLayer = [CALayer layer];
	squareLayer.cornerRadius = kSquareCornerRadius;
	squareLayer.bounds = CGRectMake(0.0f, 0.0f, kSquareWidth, kSquareWidth);
	squareLayer.position = CGPointMake(kSquareWidth / 2, kSquareWidth / 2);
	squareLayer.backgroundColor = CGColorCreateGenericRGB(1.0f, 0.1f, 0.1f, 1.0f);
	
	[squareLayer setValue:[NSNumber numberWithFloat:0.01f] forKeyPath:@"transform.scale"];
	
	[rootLayer addSublayer:squareLayer];
	
	[self performSelector:@selector(startChain) withObject:nil afterDelay:0.0f];
}

- (void)startChain {
	[chainQueue addOperation:[AnimationStepOperation stepWithTarget:self action:@selector(enter:)]];
	[chainQueue addOperation:[AnimationStepOperation stepWithTarget:self action:@selector(changeColor:)]];
	[chainQueue addOperation:[AnimationStepOperation stepWithTarget:self action:@selector(rotate:)]];
}

- (void)enter:(NSOperation *)op {
	CABasicAnimation *pos = [CABasicAnimation animationWithKeyPath:@"position"];
	pos.toValue = [NSValue valueWithPoint:NSMakePoint(NSMidX([customView bounds]), NSMidY([customView bounds]))];
	
	CABasicAnimation *scale = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
	scale.toValue = [NSNumber numberWithFloat:1.0f];
	
	CAAnimationGroup *group = [CAAnimationGroup animation];
	group.animations = [NSArray arrayWithObjects:pos, scale, nil];
	group.fillMode = kCAFillModeForwards;
	group.removedOnCompletion = NO;
	group.duration = 0.75f;
	group.delegate = op;
	
	[squareLayer addAnimation:group forKey:@"enter"];
}

- (void)changeColor:(NSOperation *)op {
	CABasicAnimation *color = [CABasicAnimation animationWithKeyPath:@"backgroundColor"];
	color.toValue = (id) CGColorCreateGenericRGB(0.2f, 0.8f, 0.3f, 1.0f);
	color.fillMode = kCAFillModeForwards;
	color.removedOnCompletion = NO;
	color.duration = 0.5f;
	color.delegate = op;
	
	[squareLayer addAnimation:color forKey:@"color"];
}

- (void)rotate:(NSOperation *)op {
	CABasicAnimation *rotate = [CABasicAnimation animationWithKeyPath:@"transform.rotation.z"];
	rotate.toValue = [NSNumber numberWithDouble:-M_PI];
	rotate.repeatCount = 2;
	rotate.fillMode = kCAFillModeForwards;
	rotate.removedOnCompletion = NO;
	rotate.duration = 0.5f;
	rotate.delegate = op;
	
	[squareLayer addAnimation:rotate forKey:@"rotate"];
}

@end
