//
//  AnimationStepOperation.m
//  AnimationChain
//
//  Copyright Matt Rajca 2009. All rights reserved.
//

#import "AnimationStepOperation.h"

@implementation AnimationStepOperation

+ (id)stepWithTarget:(id)aTarget action:(SEL)aSelector {
	return [[[self alloc] initWithTarget:aTarget action:aSelector] autorelease];
}

- (id)initWithTarget:(id)aTarget action:(SEL)aSelector {
	self = [super init];
	if (self) {
		target = aTarget;
		selector = aSelector;
	}
	return self;
}

- (BOOL)isConcurrent {
	return YES;
}

- (BOOL)isExecuting {
	return !done;
}

- (BOOL)isFinished {
	return done;
}

- (void)start {
	[target performSelectorOnMainThread:selector withObject:self waitUntilDone:NO];
}

- (void)animationDidStop:(CAAnimation *)anim finished:(BOOL)flag {
	[self willChangeValueForKey:@"isExecuting"];
	[self willChangeValueForKey:@"isFinished"];
	
	done = YES;
	
	[self didChangeValueForKey:@"isExecuting"];
	[self didChangeValueForKey:@"isFinished"];
}

@end
